@@ -127,16 +127,16 @@ class WebSocketApp(object):
         self.header = header if header is not None else []
         self.cookie = cookie
 
-        self.on_open = on_open or getattr(self, 'on_open', None)
-        self.on_message = on_message or getattr(self, 'on_message', None)
-        self.on_data = on_data or getattr(self, 'on_data', None)
-        self.on_error = on_error or getattr(self, 'on_error', None)
-        self.on_close = on_close or getattr(self, 'on_close', None)
-        self.on_ping = on_ping or getattr(self, 'on_ping', None)
-        self.on_pong = on_pong or getattr(self, 'on_pong', None)
-        self.on_cont_message = on_cont_message or getattr(self, 'on_cont_message', None)
-        self.get_mask_key = get_mask_key or getattr(self, 'get_mask_key', None)
+        self.on_open = on_open
+        self.on_message = on_message
+        self.on_data = on_data
+        self.on_error = on_error
+        self.on_close = on_close
+        self.on_ping = on_ping
+        self.on_pong = on_pong
+        self.on_cont_message = on_cont_message
         self.keep_running = False
+        self.get_mask_key = get_mask_key
         self.sock = None
         self.last_ping_tm = 0
         self.last_pong_tm = 0
@@ -282,9 +282,14 @@ class WebSocketApp(object):
                 return True
 
             def check():
-                if ping_timeout and self.last_ping_tm \
-                        and time.time() - self.last_ping_tm > ping_timeout \
-                        and self.last_ping_tm - self.last_pong_tm > ping_timeout:
+                has_timeout_expired = time.time() - self.last_ping_tm > ping_timeout
+                has_pong_not_arrived_after_last_ping = self.last_pong_tm - self.last_ping_tm < 0
+                has_pong_arrived_too_late = self.last_pong_tm - self.last_ping_tm > ping_timeout
+
+                if (ping_timeout
+                        and self.last_ping_tm
+                        and has_timeout_expired
+                        and (has_pong_not_arrived_after_last_ping or has_pong_arrived_too_late)):
                     raise WebSocketTimeoutException("ping/pong timed out")
                 return True
 
@@ -306,7 +311,6 @@ class WebSocketApp(object):
     def _get_close_args(self, data):
         """ this functions extracts the code, reason from the close body
         if they exists, and if the self.on_close except three arguments """
-        import inspect
         # if the on_close callback is "old", just return empty list
         if sys.version_info < (3, 0):
             if not self.on_close or len(inspect.getargspec(self.on_close).args) != 3:
